##
##  Track remote clients.
##
package POE::Jukebox::ClientManager;
use strict;

use POE qw(Driver::SysRW Wheel::ReadWrite Filter::Stream);

sub new
{
	my $self = shift;
	my @list = @_;
	
	POE::Session->create(
		args => [ @list ],
		inline_states => {
			_start => \&_start,
			_stop => \&_stop,
			new_client => \&new_client,
			dead_client => \&dead_client,
			write_all=> \&write_all,	
			register_for=> \&register_for,	
			unregister_for=> \&unregister_for,	
			write_category=> \&write_category,	
		},
	);

	print "Created Client Manager\n";
}

sub _start
{
	print "Starting Client Manager\n";
	
	$_[KERNEL]->alias_set( 'clientmanager' );

	## Set up internal storage.
	$_[HEAP]->{clients} = [];

	## Set up legal categories for messages.
	$_[HEAP]->{categories}->{update} = [];
	$_[HEAP]->{categories}->{'progress'} = [];

	print "Started Client Manager\n";
}

sub _stop
{
	delete $_[HEAP]->{self};
	print "Stopped Client Manager\n";
}

## Add client to list.
sub new_client
{
	my $session = $_[ARG0];
	push @{$_[HEAP]->{clients}}, $session;
}

## Remove client from list.
sub dead_client
{
	my $session = $_[ARG0];
	return unless $session;
	@{$_[HEAP]->{clients}} = grep { $_ ne $session } @{$_[HEAP]->{clients}};
	foreach my $key (keys %{$_[HEAP]->{categories}}) {
		$_[KERNEL]->yield('unregister_for',$session,$key);
	}
	
}

sub write_all
{
	my $msg = $_[ARG0];
	foreach my $session (@{$_[HEAP]->{clients}}) {
		$_[KERNEL]->post($session, 'out_to_client',$msg);
	}
}

sub register_for
{
	my $session = $_[ARG0];
	my $msg_category = $_[ARG1];
	if(exists($_[HEAP]->{categories}->{$msg_category})) {
		push @{$_[HEAP]->{categories}->{$msg_category}}, $session;
		$_[KERNEL]->post($session,'out_to_client',"OK");
	} else {
		$_[KERNEL]->post($session,'out_to_client',"ERR: No such message category");
	}
}

sub unregister_for
{
	my $session = $_[ARG0];
	my $msg_category = $_[ARG1];
	die "No session" unless $session;
	die "No msg_category" unless $msg_category;
	if(defined($_[HEAP]->{categories}->{$msg_category})) {
		@{$_[HEAP]->{categories}->{$msg_category}} = grep { $_ != $session } @{$_[HEAP]->{categories}->{$msg_category}};
		$_[KERNEL]->post($session,'out_to_client',"OK");
	} else {
		$_[KERNEL]->post($session,'out_to_client',"ERR: No such message category");
	}
}

sub write_category
{
	my $category = $_[ARG0];
	my $msg = $_[ARG1];

	die "No such message category" unless exists($_[HEAP]->{categories}->{$category});
	
	foreach my $session (@{$_[HEAP]->{categories}->{$category}}) {
		$_[KERNEL]->post($session, 'out_to_client',$msg);
	}
}

1;
