## 
use strict;
package POE::Jukebox::SimplePlaylist;

use POE qw(Driver::SysRW Wheel::ReadWrite Filter::Stream);

use IO::File;

sub new
{
	my $self = shift;
	my $list = shift;
	
	POE::Session->create(
		args => [ $list ],
		inline_states => {
			_start => \&_start,
			_stop => \&_stop,
			songcomplete => \&playlist_song_complete,
			next => \&playlist_next,
			prev => \&playlist_prev,
		},
	);

	print "Created playlist\n";
}

sub _start
{
	my $listfile = $_[ARG0];
	
	print "Starting playlist\n";
	
	$_[KERNEL]->alias_set( 'playlist' );
	
	print "Opening file $listfile\n";
	my $fh = new IO::File $listfile, "r";

	if(!defined $fh) {
		## no file, no go.
		return;
	} 
	
	## Read in listfile.
	print "Reading file $listfile\n";
	while(my $line = <$fh>) {
		$line =~ s/\n//;		
		next unless $line =~ /\.ogg$/;
		push @{$_[HEAP]->{playlist}},$line;
	}

	## Shuffle playlist
	fisher_yates_shuffle($_[HEAP]->{playlist});
	
	## Set current pos.
	$_[HEAP]->{current_song} = 0;

	print "Started playlist\n";

	## Set up signals.
	## TODO: Write better control method!
	$_[KERNEL]->sig('USR2' => 'next');
	$_[KERNEL]->sig('USR1' => 'prev');
	

	$_[KERNEL]->yield('songcomplete');
}

sub _stop
{
	delete $_[HEAP]->{self};
	print "Stopped playlist\n";
}

sub playlist_song_complete
{
	print "Moving to next song\n";
	$_[HEAP]->{current_song}++;
	print "posting song $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}] to player\n";
	$_[KERNEL]->post('player','song_new', $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}]);
}

sub playlist_next
{
	print "Skipping to next song\n";
	
	$_[HEAP]->{current_song}++;
	print "posting song $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}] to player\n";
	$_[KERNEL]->post('player','song_stop');
	$_[KERNEL]->post('player','song_new', $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}]);
}

sub playlist_prev
{
	print "Skipping to previous song\n";
	$_[HEAP]->{current_song}--;
	print "posting song $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}] to player\n";
	$_[KERNEL]->post('player','song_stop');
	$_[KERNEL]->post('player','song_new', $_[HEAP]->{playlist}->[$_[HEAP]->{current_song}]);
}

sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
	
	print "Shuffling playlist\n";
	
    for ($i = @$array; $i--;) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i]
    }
}

1;
