use strict;

package POE::Jukebox::TcpConnection;

use POE qw(Driver::SysRW Wheel::ReadWrite Filter::Stream Filter::Line);

sub new
{
	my $class = shift;
	my $self = bless({}, $class);
		
	my $sock = shift;
	my $addr = shift;
	my $wheel_id = shift;

	POE::Session->create(
		args => [ $sock, $addr ],
		heap => {
			socket => $sock,
			wheel_id => $sock,
			remote_addr => $addr || 'unknown',
		},
		inline_states => {
			_start => \&_start,
			_stop => \&_stop,
			in_from_client => \&client_input,
			out_to_client => \&client_output,
			got_error => \&got_error,
			
		},
	);

	print "Created TcpConnection ($addr)\n";
}

sub _start
{
	my $socket = $_[ARG0];
	$_[HEAP]->{wheel} = POE::Wheel::ReadWrite->new(
		Handle => $socket,
		Driver => new POE::Driver::SysRW(),
		OutputFilter => new POE::Filter::Stream(),
		InputFilter => new POE::Filter::Line(),
		InputState => 'in_from_client',
		ErrorState => 'got_error',
	);

	## Tell whoever is tracking sessions that we're alive.
	$_[KERNEL]->post('clientmanager', 'new_client', $_[SESSION]);
}

sub _stop
{
	## Tell whoever is tracking sessions that we're dead.
	print STDERR "Session ", $_[SESSION]->ID, " going away now...\n";
	$_[KERNEL]->post('clientmanager', 'dead_client', $_[SESSION]);
	delete($_[HEAP]->{wheel});
	delete $_[HEAP]->{self};	
}

sub client_input 
{
	my ($command,@args) = split(/\s+/,$_[ARG0]);

	print "Received command '$command' with args [", join(' ',@args), "] on session ",$_[SESSION]->ID,"\n";

	## Here, we handle commands and figure out what to do with them.
	## TODO: Validate commands and send to appropriate places
	## TODO: 
	for($command) {
		/^quit$/i && do { 
			$_[KERNEL]->yield('_stop'); last;
		};
		/^mode$/i && do { $_[KERNEL]->post('playlist','mode',$_[SESSION],@args); last;};
		/^next$/i && do { $_[KERNEL]->post('playlist','next',$_[SESSION]); last; };
		/^prev$/i && do { $_[KERNEL]->post('playlist','prev',$_[SESSION]); last; };
		/^pause$/i && do { $_[KERNEL]->post('playlist','pause',$_[SESSION]); last; };
		/^volume$/i && do { $_[KERNEL]->post('audiomanager','volume',$_[SESSION],@args); last; };
		/^current$/i && do { $_[KERNEL]->post('playlist','current',$_[SESSION]); last; };
		/^status$/i && do { $_[KERNEL]->post('playlist','current',$_[SESSION]); last; };
		/^register_for$/i && do { $_[KERNEL]->post('clientmanager','register_for',$_[SESSION],$args[0]); last; };
		/^unregister_for$/i && do { $_[KERNEL]->post('clientmanager','unregister_for',$_[SESSION],$args[0]); last; };

		$_[KERNEL]->yield('out_to_client',"ERR No such command");
	}
	
}

sub got_error
{
	my ($operation, $errnum, $errstr) = @_[ARG0, ARG1, ARG2];
	print "$_[HEAP]->{remote_addr} detected $operation error $errnum: $errstr\n";
	if($operation eq 'read') {
		# Disconnect on read error.
		$_[KERNEL]->yield('_stop');
	}
}

sub client_output 
{
	my $data = $_[ARG0];
##	print "Sending [$data] to client $_[HEAP]->{remote_addr}\n";
	$_[HEAP]->{wheel}->put("$data\n");
}

1;
