#!/usr/bin/perl -w

##  Grab all song info from the discinfo.xml files
##  and populate our db with it.
use strict;

use File::Basename;
use File::Find;
use XML::Simple;

use MLDBM qw(DB_File Storable);

## Directories potentially containing .xml files describing ogg info
## are in @ARGV
my @dirs;
foreach my $dir (@ARGV) {
	push @dirs, $dir if (-d $dir);
}


my %songhash;
tie(%songhash, 'MLDBM','songdata.db') or die $!;

my %albumhash;
tie(%albumhash, 'MLDBM','albumdata.db') or die $!;

my %artisthash;
tie(%artisthash, 'MLDBM','artistdata.db') or die $!;

my %genrehash;
tie(%genrehash, 'MLDBM','genredata.db') or die $!;

my @files;
find( { 
	wanted => sub { /^.*\.xml\z/s && push @files, $File::Find::name; },
	no_chdir => 1
}, @dirs);

foreach my $file (@files) {
	print "Processing $file\n";
	my $dir = dirname($file);	
	my $album = XMLin(
		$file,
		suppressempty => 1,
		keyattr => {},
		forcearray => [ 'track' ]);
	print "Found album $album->{title}\n";
	
	## Check for uuid...
	if(!exists($album->{uuid})) {
		print "No UUID! Skipping!\n";
		next;
	}
	
	my $album_length;
	my $album_info = {
		title => $album->{title},
		genre => $album->{genre},
		tracks => {},
	};
	if ($album->{type} eq 'single-artist') {
		$album_info->{artist} = $album->{artist}->{content};
	} else {
		$album_info->{artist} = 'Multiple Artists';
	}
	foreach my $song (@{$album->{tracklist}->{track}}) {
		my $artist;
		if($album->{type} eq 'single-artist') {
			$artist = $album->{artist}->{content};
		} else {
			$artist = $song->{artist}->{content};
		}
		my $song_info = {
			uuid => $song->{uuid},
			track_title => $song->{title},
			track_artist => $artist,
			album_title => $album->{title},
			album_uuid => $album->{uuid},
			number => $song->{'track-number'},
			genre => $album->{genre},
			'length' => $song->{'length'}->{content},
			filename => "$dir/$song->{filename}",
		};
		$album_info->{tracks}->{$song->{'track-number'}} = $song->{uuid};
		$album_length += $song->{'length'}->{content};
		$songhash{$song->{uuid}} = $song_info;
	
		my $tmp; 
		if( defined($genrehash{$album->{genre}}) ) {
			$tmp = $genrehash{$album->{genre}};
		} else {
			$tmp = [ ];
		}
		push @$tmp, $song->{uuid};
		$genrehash{$album->{genre}} = $tmp;

		if( defined($artisthash{$artist}) ) {
			$tmp = $artisthash{$artist};
		} else {
			$tmp = [ ];
		}
		push @$tmp, $song->{uuid};
		$artisthash{$artist} = $tmp;
	}
	$album_info->{'length'} = $album_length;
	$albumhash{$album->{uuid}} = $album_info;
}

untie %albumhash;
untie %songhash;
untie %genrehash;
untie %artisthash;
